<?php
class Thumbnail
{

		public $errmsg = NULL;
		public $error = NULL;
		public $format = NULL;
		public $fileName = NULL;
		public $imageMeta = NULL;
		public $currentDimensions = NULL;
		public $newDimensions = NULL;
		public $newImage = NULL;
		public $oldImage = NULL;
		public $workingImage = NULL;
		public $percent = NULL;
		public $maxWidth = NULL;
		public $maxHeight = NULL;

		public function Thumbnail( $fileName )
		{
				if ( !function_exists( "gd_info" ) )
				{
						echo "You do not have the GD Library installed.  This class requires the GD library to function properly.\n";
						echo "visit http://us2.php.net/manual/en/ref.image.php for more information";
						exit( );
				}
				$this->errmsg = "";
				$this->error = FALSE;
				$this->currentDimensions = array( );
				$this->newDimensions = array( );
				$this->fileName = $fileName;
				$this->imageMeta = array( );
				$this->percent = 100;
				$this->maxWidth = 0;
				$this->maxHeight = 0;
				if ( !file_exists( $this->fileName ) )
				{
						$this->errmsg = "File not found";
						$this->error = TRUE;
				}
				else if ( !is_readable( $this->fileName ) )
				{
						$this->errmsg = "File is not readable";
						$this->error = TRUE;
				}
				if ( $this->error == FALSE )
				{
						if ( stristr( strtolower( $this->fileName ), ".gif" ) )
						{
								$this->format = "GIF";
						}
						else if ( stristr( strtolower( $this->fileName ), ".jpg" ) || stristr( strtolower( $this->fileName ), ".jpeg" ) )
						{
								$this->format = "JPG";
						}
						else if ( stristr( strtolower( $this->fileName ), ".png" ) )
						{
								$this->format = "PNG";
						}
						else
						{
								$this->errmsg = "Unknown file format";
								$this->error = TRUE;
						}
				}
				if ( $this->error == FALSE )
				{
						switch ( $this->format )
						{
						case "GIF" :
								$this->oldImage = imagecreatefromgif( $this->fileName );
								break;
						case "JPG" :
								$this->oldImage = imagecreatefromjpeg( $this->fileName );
								break;
						case "PNG" :
								$this->oldImage = imagecreatefrompng( $this->fileName );
								break;
						}
						$size = getimagesize( $this->fileName );
						$this->currentDimensions = array(
								"width" => $size[0],
								"height" => $size[1]
						);
						$this->newImage = $this->oldImage;
						$this->gatherImageMeta( );
				}
				if ( $this->error == TRUE )
				{
						$this->showErrorImage( );
						break;
				}
		}

		public function destruct( )
		{
				if ( is_resource( $this->newImage ) )
				{
						@imagedestroy( $this->newImage );
				}
				if ( is_resource( $this->oldImage ) )
				{
						@imagedestroy( $this->oldImage );
				}
				if ( is_resource( $this->workingImage ) )
				{
						@imagedestroy( $this->workingImage );
				}
		}

		public function getCurrentWidth( )
		{
				return $this->currentDimensions['width'];
		}

		public function getCurrentHeight( )
		{
				return $this->currentDimensions['height'];
		}

		public function calcWidth( $width, $height )
		{
				$newWp = 100 * $this->maxWidth / $width;
				$newHeight = $height * $newWp / 100;
				return array(
						"newWidth" => intval( $this->maxWidth ),
						"newHeight" => intval( $newHeight )
				);
		}

		public function calcHeight( $width, $height )
		{
				$newHp = 100 * $this->maxHeight / $height;
				$newWidth = $width * $newHp / 100;
				return array(
						"newWidth" => intval( $newWidth ),
						"newHeight" => intval( $this->maxHeight )
				);
		}

		public function calcPercent( $width, $height )
		{
				$newWidth = $width * $this->percent / 100;
				$newHeight = $height * $this->percent / 100;
				return array(
						"newWidth" => intval( $newWidth ),
						"newHeight" => intval( $newHeight )
				);
		}

		public function calcImageSize( $width, $height )
		{
				$newSize = array(
						"newWidth" => $width,
						"newHeight" => $height
				);
				if ( 0 < $this->maxWidth )
				{
						$newSize = $this->calcWidth( $width, $height );
						if ( 0 < $this->maxHeight && $this->maxHeight < $newSize['newHeight'] )
						{
								$newSize = $this->calcHeight( $newSize['newWidth'], $newSize['newHeight'] );
						}
				}
				if ( 0 < $this->maxHeight )
				{
						$newSize = $this->calcHeight( $width, $height );
						if ( 0 < $this->maxWidth && $this->maxWidth < $newSize['newWidth'] )
						{
								$newSize = $this->calcWidth( $newSize['newWidth'], $newSize['newHeight'] );
						}
				}
				$this->newDimensions = $newSize;
		}

		public function calcImageSizePercent( $width, $height )
		{
				if ( 0 < $this->percent )
				{
						$this->newDimensions = $this->calcPercent( $width, $height );
				}
		}

		public function showErrorImage( )
		{
				header( "Content-type: image/png" );
				$errImg = imagecreate( 220, 25 );
				$bgColor = imagecolorallocate( $errImg, 0, 0, 0 );
				$fgColor1 = imagecolorallocate( $errImg, 255, 255, 255 );
				$fgColor2 = imagecolorallocate( $errImg, 255, 0, 0 );
				imagestring( $errImg, 3, 6, 6, "Error:", $fgColor2 );
				imagestring( $errImg, 3, 55, 6, $this->errmsg, $fgColor1 );
				imagepng( $errImg );
				imagedestroy( $errImg );
		}

		public function resize( $maxWidth = 0, $maxHeight = 0 )
		{
				$this->maxWidth = $maxWidth;
				$this->maxHeight = $maxHeight;
				$this->calcImageSize( $this->currentDimensions['width'], $this->currentDimensions['height'] );
				if ( function_exists( "ImageCreateTrueColor" ) )
				{
						$this->workingImage = imagecreatetruecolor( $this->newDimensions['newWidth'], $this->newDimensions['newHeight'] );
				}
				else
				{
						$this->workingImage = imagecreate( $this->newDimensions['newWidth'], $this->newDimensions['newHeight'] );
				}
				imagecopyresampled( $this->workingImage, $this->oldImage, 0, 0, 0, 0, $this->newDimensions['newWidth'], $this->newDimensions['newHeight'], $this->currentDimensions['width'], $this->currentDimensions['height'] );
				$this->oldImage = $this->workingImage;
				$this->newImage = $this->workingImage;
				$this->currentDimensions['width'] = $this->newDimensions['newWidth'];
				$this->currentDimensions['height'] = $this->newDimensions['newHeight'];
		}

		public function md_resize_to_new_size( $maxWidth = 0, $maxHeight = 0 )
		{
				$this->maxWidth = $maxWidth;
				$this->maxHeight = $maxHeight;
				$this->md_calcImageSize( $this->currentDimensions['width'], $this->currentDimensions['height'] );
				if ( function_exists( "ImageCreateTrueColor" ) )
				{
						$this->workingImage = imagecreatetruecolor( $this->newDimensions['newWidth'], $this->newDimensions['newHeight'] );
				}
				else
				{
						$this->workingImage = imagecreate( $this->newDimensions['newWidth'], $this->newDimensions['newHeight'] );
				}
				imagecopyresampled( $this->workingImage, $this->oldImage, 0, 0, 0, 0, $this->newDimensions['newWidth'], $this->newDimensions['newHeight'], $this->currentDimensions['width'], $this->currentDimensions['height'] );
				$this->oldImage = $this->workingImage;
				$this->newImage = $this->workingImage;
				$this->currentDimensions['width'] = $this->newDimensions['newWidth'];
				$this->currentDimensions['height'] = $this->newDimensions['newHeight'];
		}

		public function md_calcImageSize( $width, $height )
		{
				$newSize['newWidth'] = $this->maxWidth;
				$newSize['newHeight'] = $this->maxHeight;
				$this->newDimensions = $newSize;
		}

		public function resizePercent( $percent = 0 )
		{
				$this->percent = $percent;
				$this->calcImageSizePercent( $this->currentDimensions['width'], $this->currentDimensions['height'] );
				if ( function_exists( "ImageCreateTrueColor" ) )
				{
						$this->workingImage = imagecreatetruecolor( $this->newDimensions['newWidth'], $this->newDimensions['newHeight'] );
				}
				else
				{
						$this->workingImage = imagecreate( $this->newDimensions['newWidth'], $this->newDimensions['newHeight'] );
				}
				imagecopyresampled( $this->workingImage, $this->oldImage, 0, 0, 0, 0, $this->newDimensions['newWidth'], $this->newDimensions['newHeight'], $this->currentDimensions['width'], $this->currentDimensions['height'] );
				$this->oldImage = $this->workingImage;
				$this->newImage = $this->workingImage;
				$this->currentDimensions['width'] = $this->newDimensions['newWidth'];
				$this->currentDimensions['height'] = $this->newDimensions['newHeight'];
		}

		public function cropFromCenter( $cropSize )
		{
				if ( $this->currentDimensions['width'] < $cropSize )
				{
						$cropSize = $this->currentDimensions['width'];
				}
				if ( $this->currentDimensions['height'] < $cropSize )
				{
						$cropSize = $this->currentDimensions['height'];
				}
				$cropX = intval( ( $this->currentDimensions['width'] - $cropSize ) / 2 );
				$cropY = intval( ( $this->currentDimensions['height'] - $cropSize ) / 2 );
				if ( function_exists( "ImageCreateTrueColor" ) )
				{
						$this->workingImage = imagecreatetruecolor( $cropSize, $cropSize );
				}
				else
				{
						$this->workingImage = imagecreate( $cropSize, $cropSize );
				}
				imagecopyresampled( $this->workingImage, $this->oldImage, 0, 0, $cropX, $cropY, $cropSize, $cropSize, $cropSize, $cropSize );
				$this->oldImage = $this->workingImage;
				$this->newImage = $this->workingImage;
				$this->currentDimensions['width'] = $cropSize;
				$this->currentDimensions['height'] = $cropSize;
		}

		public function crop( $startX, $startY, $width, $height )
		{
				if ( $this->currentDimensions['width'] < $width )
				{
						$width = $this->currentDimensions['width'];
				}
				if ( $this->currentDimensions['height'] < $height )
				{
						$height = $this->currentDimensions['height'];
				}
				if ( $this->currentDimensions['width'] < $startX + $width )
				{
						$startX = $this->currentDimensions['width'] - $width;
				}
				if ( $this->currentDimensions['height'] < $startY + $height )
				{
						$startY = $this->currentDimensions['height'] - $height;
				}
				if ( $startX < 0 )
				{
						$startX = 0;
				}
				if ( $startY < 0 )
				{
						$startY = 0;
				}
				if ( function_exists( "ImageCreateTrueColor" ) )
				{
						$this->workingImage = imagecreatetruecolor( $width, $height );
				}
				else
				{
						$this->workingImage = imagecreate( $width, $height );
				}
				imagecopyresampled( $this->workingImage, $this->oldImage, 0, 0, $startX, $startY, $width, $height, $width, $height );
				$this->oldImage = $this->workingImage;
				$this->newImage = $this->workingImage;
				$this->currentDimensions['width'] = $width;
				$this->currentDimensions['height'] = $height;
		}

		public function show( $quality = 100, $name = "" )
		{
				switch ( $this->format )
				{
				case "GIF" :
						if ( $name != "" )
						{
								imagegif( $this->newImage, $name );
						}
						else
						{
								header( "Content-type: image/gif" );
								imagegif( $this->newImage );
						}
						break;
				case "JPG" :
						if ( $name != "" )
						{
								imagejpeg( $this->newImage, $name, $quality );
						}
						else
						{
								header( "Content-type: image/jpeg" );
								imagejpeg( $this->newImage, "", $quality );
						}
						break;
				case "PNG" :
						if ( $name != "" )
						{
								imagepng( $this->newImage, $name );
						}
						else
						{
								header( "Content-type: image/png" );
								imagepng( $this->newImage );
						}
				}
		}

		public function save( $name, $quality = 100 )
		{
				$this->show( $quality, $name );
		}

		public function createReflection( $percent, $reflection, $white, $border = TRUE, $borderColor = "#a4a4a4" )
		{
				$width = $this->currentDimensions['width'];
				$height = $this->currentDimensions['height'];
				$reflectionHeight = intval( $height * ( $reflection / 100 ) );
				$newHeight = $height + $reflectionHeight;
				$reflectedPart = $height * ( $percent / 100 );
				$this->workingImage = imagecreatetruecolor( $width, $newHeight );
				imagealphablending( $this->workingImage, TRUE );
				$colorToPaint = imagecolorallocatealpha( $this->workingImage, 255, 255, 255, 0 );
				imagefilledrectangle( $this->workingImage, 0, 0, $width, $newHeight, $colorToPaint );
				imagecopyresampled( $this->workingImage, $this->newImage, 0, 0, 0, $reflectedPart, $width, $reflectionHeight, $width, $height - $reflectedPart );
				$this->imageFlipVertical( );
				imagecopy( $this->workingImage, $this->newImage, 0, 0, 0, 0, $width, $height );
				imagealphablending( $this->workingImage, TRUE );
				$i = 0;
				for ( ;	$i < $reflectionHeight;	++$i	)
				{
						$colorToPaint = imagecolorallocatealpha( $this->workingImage, 255, 255, 255, ( $i / $reflectionHeight * -1 + 1 ) * $white );
						imagefilledrectangle( $this->workingImage, 0, $height + $i, $width, $height + $i, $colorToPaint );
				}
				if ( $border )
				{
						$rgb = $this->hex2rgb( $borderColor, FALSE );
						$colorToPaint = imagecolorallocate( $this->workingImage, $rgb[0], $rgb[1], $rgb[2] );
						imageline( $this->workingImage, 0, 0, $width, 0, $colorToPaint );
						imageline( $this->workingImage, 0, $height, $width, $height, $colorToPaint );
						imageline( $this->workingImage, 0, 0, 0, $height, $colorToPaint );
						imageline( $this->workingImage, $width - 1, 0, $width - 1, $height, $colorToPaint );
				}
				$this->oldImage = $this->workingImage;
				$this->newImage = $this->workingImage;
				$this->currentDimensions['width'] = $width;
				$this->currentDimensions['height'] = $newHeight;
		}

		public function imageFlipVertical( )
		{
				$x_i = imagesx( $this->workingImage );
				$y_i = imagesy( $this->workingImage );
				$x = 0;
				for ( ;	$x < $x_i;	++$x	)
				{
						$y = 0;
						for ( ;	$y < $y_i;	++$y	)
						{
								imagecopy( $this->workingImage, $this->workingImage, $x, $y_i - $y - 1, $x, $y, 1, 1 );
						}
				}
		}

		public function hex2rgb( $hex, $asString = FALSE )
		{
				if ( 0 === strpos( $hex, "#" ) )
				{
						$hex = substr( $hex, 1 );
				}
				else if ( 0 === strpos( $hex, "&H" ) )
				{
						$hex = substr( $hex, 2 );
				}
				$cutpoint = ceil( strlen( $hex ) / 2 ) - 1;
				$rgb = explode( ":", wordwrap( $hex, $cutpoint, ":", $cutpoint ), 3 );
				$rgb[0] = isset( $rgb[0] ) ? hexdec( $rgb[0] ) : 0;
				$rgb[1] = isset( $rgb[1] ) ? hexdec( $rgb[1] ) : 0;
				$rgb[2] = isset( $rgb[2] ) ? hexdec( $rgb[2] ) : 0;
				if ( $asString )
				{
						return "{$rgb[0]} {$rgb[1]} {$rgb[2]}";
				}
				return $rgb;
		}

		public function gatherImageMeta( )
		{
				if ( function_exists( "exif_read_data" ) && $this->format == "JPG" )
				{
						$imageData = exif_read_data( $this->fileName );
						if ( isset( $imageData['Make'] ) )
						{
								$this->imageMeta['make'] = ucwords( strtolower( $imageData['Make'] ) );
						}
						if ( isset( $imageData['Model'] ) )
						{
								$this->imageMeta['model'] = $imageData['Model'];
						}
						if ( isset( $imageData['COMPUTED']['ApertureFNumber'] ) )
						{
								$this->imageMeta['aperture'] = $imageData['COMPUTED']['ApertureFNumber'];
								$this->imageMeta['aperture'] = str_replace( "/", "", $this->imageMeta['aperture'] );
						}
						if ( isset( $imageData['ExposureTime'] ) )
						{
								$exposure = explode( "/", $imageData['ExposureTime'] );
								$exposure = round( $exposure[1] / $exposure[0], -1 );
								$this->imageMeta['exposure'] = "1/".$exposure." second";
						}
						if ( isset( $imageData['Flash'] ) )
						{
								if ( 0 < $imageData['Flash'] )
								{
										$this->imageMeta['flash'] = "Yes";
								}
								else
								{
										$this->imageMeta['flash'] = "No";
								}
						}
						if ( isset( $imageData['FocalLength'] ) )
						{
								$focus = explode( "/", $imageData['FocalLength'] );
								$this->imageMeta['focalLength'] = round( $focus[0] / $focus[1], 2 )." mm";
						}
						if ( isset( $imageData['DateTime'] ) )
						{
								$date = $imageData['DateTime'];
								$date = explode( " ", $date );
								$date = str_replace( ":", "-", $date[0] )." ".$date[1];
								$this->imageMeta['dateTaken'] = date( "m/d/Y g:i A", strtotime( $date ) );
						}
				}
		}

		public function rotateImage( $direction = "CW" )
		{
				if ( $direction == "CW" )
				{
						$this->workingImage = imagerotate( $this->workingImage, -90, 0 );
				}
				else
				{
						$this->workingImage = imagerotate( $this->workingImage, 90, 0 );
				}
				$newWidth = $this->currentDimensions['height'];
				$newHeight = $this->currentDimensions['width'];
				$this->oldImage = $this->workingImage;
				$this->newImage = $this->workingImage;
				$this->currentDimensions['width'] = $newWidth;
				$this->currentDimensions['height'] = $newHeight;
		}

}

?>